# This file is part of the Falcon repository manager
# Copyright (C) 2005-2007 Dennis Kaarsemaker
# See the file named COPYING in the root of the source tree for license details

# Set up bugcatching very early. When crashes occur, reset the terminal if it changed
import sys, termios, errno, os, traceback
termattr = termios.tcgetattr(0)
def except_hook(etype, evalue, etraceback):
    new_termattr = termios.tcgetattr(0)
    if new_termattr != termattr:
        termios.tcsetattr(0,termios.TCSANOW, termattr)
        os.system('tput init')
    if issubclass(etype, SystemExit) or issubclass(etype, KeyboardInterrupt):
        return
    if '_' not in locals():
        _ = lambda x: x
    if issubclass(etype, OSError) and evalue.args[0] == errno.EACCES:
        print _("E: You appear to lack privileges to manage this repository")
        print str(evalue)
        return

    trace = traceback.format_exception(etype, evalue, etraceback)
    print _("""\n === ERROR! ===

An error occured in %s. This is probably a bug in the software.
Please file a bug at https://launchpad.net/falcon/+filebug and
include the following backtrace:
""" % os.path.basename(sys.argv[0]))
    print ''.join(trace)
sys.excepthook = except_hook

import falcon.config
# falcon.config initializes the django configuration as well, so we can now import everything
import falcon.plugin as plugin
import falcon.util as util
import falcon.iso as iso

import fcntl
from gettext import gettext as _
# Add our picklefield
from django.db import models as _models, get_creation_module
from django.db.models.fields import Field
import cPickle as pickle

class PickleField(Field):
    def to_python(self, value):
        if type(value) == str:
            return pickle.loads(value)
        return value
    def get_db_prep_lookup(self, value):
        return Field.get_db_prep_lookup(self, pickle.dumps(value))
    def get_db_prep_save(self, value):
        return Field.get_db_prep_save(self, pickle.dumps(value))
_models.PickleField = PickleField
m = get_creation_module()
m.DATA_TYPES['PickleField'] = 'text'

# These need the picklefield, so import after defining it
import falcon.pocket as pocket
import falcon.package as package
import falcon.mirror as mirror

lockfd = None
screen = None
def lock():
    global lockfd
    lockfd = open(os.path.join('.falcon','lock'),'a')
    try:
        fcntl.flock(lockfd,fcntl.LOCK_EX | fcntl.LOCK_NB)
    except IOError:
        util.error(_("Repository is locked by another process"))

def unlock():
    global lockfd
    if lockfd:
        fcntl.flock(lockfd,fcntl.LOCK_UN)

conf = config.conf
