# This file is part of the Falcon repository manager
# Copyright (C) 2005-2007 Dennis Kaarsemaker
# See the file named COPYING in the root of the source tree for license details

import falcon
from django import template
import re

register = template.Library()

@register.tag
def api(parser, token):
    try:
        tag_name, api_string = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError, _("%r tag requires a single argument") % token.contents.split()[0]
    try:
        api_major, api_minor = [int(x) for x in api_string.split('.')]
    except ValueError:
        raise template.TemplateSyntaxError, _("Template api should be 2 numbers, separated by a fullstop")
    if api_major != falcon.conf.template_api[0] or api_minor > falcon.conf.template_api[1]:
        falcon.util.error(_("The template '%s' was made for an incompatible version of falcon") % falcon.conf.template)
    if api_minor != falcon.conf.template_api[1]:
        falcon.util.warning(_("The template '%s' was made for an older version of falcon but is still compatible") % falcon.conf.template)
    return ApiNode()

class ApiNode(template.Node):
    def render(self, context):
        return ''

@register.filter
def format_changelog(cl):
    if not cl:
        return ''
    cl = cl[cl.find('Changes:')+8:cl.find('Files:')]
    out = '<dl>'
    cl = cl.strip().replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')
    # Strip 'empty' lines
    cl = re.sub(' (  )*\.\n','',cl)
    # Split into versions
    versions = [x for x in re.split('(?:^|\n) (?=\S)', cl, re.DOTALL | re.M) if x.strip()]
    for v in versions:
        (ver, log) = v.split("\n",1)
        out += '<b>%s</b>' % ver
        # Join the lines of items
        log = re.sub(r"\n (?:  )*([^*\-+ ])",r"\1", log)
        # Magic :)
        for line in log.split("\n"):
            l = (len(line) - len(line.lstrip()) -1) / 2
            out += '<ul><li>' * l
            out += line.lstrip()[1:]
            out += '</li></ul>' * l
        return out

@register.filter
def shortdesc(value):
    return value.split('\n',1)[0]

@register.filter
def longdesc(value):
    try:
        return '<p>%s</p>' % value.split('\n',1)[1].replace('\n .\n','</p><p>')
    except:
        return ''
